makeprobs<-function(v,N){ 
   #compute the stick-breaking weights
   probs<-v
   if(N>1){
    probs[2:N]<-probs[2:N]*cumprod(1-v[2:N-1])
   }
probs}

mpi.bcast.Robj2slave(makeprobs)


new.v<-function(g,b,m){
   v<-rep(1,m)
   for(k in 1:(m-1)){
     v[k]<-rbeta(1,sum(g==k)+1,sum(g>k)+b)
   }
v}
mpi.bcast.Robj2slave(new.v)

new.b<-function(v,a=1,b=1){
 rgamma(1,length(v)+a,b-sum(log(1-v)))}
mpi.bcast.Robj2slave(new.b)


#make a posterior draw from a wishart:
update.wish<-function(data,a,S,n,p){
  library(MCMCpack)
  S<-solve(S/(a+p))
  if(n==1){for(k1 in 1:p){for(k2 in 1:p){
     S[k1,k2]<-S[k1,k2]+sum((data[k1])*(data[k2]))
  }}}
  if(n>1){for(k1 in 1:p){for(k2 in 1:p){
     S[k1,k2]<-S[k1,k2]+sum((data[,k1])*(data[,k2]))
  }}}
#rwish(n+p+a,solve(S))}
rwish(n+p+a,solve(S))}
mpi.bcast.Robj2slave(update.wish)


MakeW<-function(d1,d2,ratio,rho,power){
 W<-1-sweep(d1+ratio*d2,2,rho^2,"/")
 W[W<0]<-0
W^power}

MakeW<-function(d1,d2,ratio,rho,power){
 W<-sweep(d1+ratio*d2,2,rho^2,"/")
exp(-0.5*W)}
mpi.bcast.Robj2slave(MakeW)



KC<-function(y,X,MISS,ZMISS,FE,s,tooth,
            commong=F,NPspatial=F,post.pred=F,
            J=10,nonstat=F,infomiss=T,spatial=T,REmissing=F,
            iters=2000,burn=1000,display=10,thin=thin){

   if(post.pred){
       y<-cbind(y,NA)
       X<-rbind(X,colMeans(X))
       MISS<-cbind(MISS,NA)
   }


   nt<-ncol(y)
   ns<-nrow(y)
   nm<-nrow(MISS)
   ntooth<-max(tooth)
   toothz<-round(as.vector(ZMISS%*%tooth))
   p<-ncol(X)
   beta1<-lm(colMeans(y,na.rm=T)~X)$coef[-1]
   beta2<-rep(0,p)


   d1<-as.matrix(dist(s[,1],upper=T,diag=T))^2
   d2<-as.matrix(dist(s[,2],upper=T,diag=T))^2
   ratio<-rho<-rep(1,ntooth)
   power<-rep(20,ntooth)
   W<-MakeW(d1,d2,ratio[tooth],rho[tooth],power[tooth])

   miss<-is.na(y)
   mny<-mean(y,na.rm=T)
   y[miss]<-mny
   taue<-rep(1,ntooth)
   z<-ifelse(MISS,1,-1) 
   L<-ifelse(MISS,0,-100)
   U<-ifelse(MISS,100, 0)
   L[is.na(L)]<--100
   U[is.na(U)]<- 100
   z[is.na(z)]<- 0

   #inital values:

   REn<-ncol(FE)
   RE1<-matrix(mny,REn,nt)
   RE2<-matrix(0,REn,nt)
   RE1mn<-matrix(mny,REn,J)
   RE1mnmn<-rep(mny,REn)

   RE2mn<-rep(0,REn)
   Avec<-rep(0,ntooth)
   A<-diag(Avec[toothz])

   RE1prec1<-RE1prec2<-RE2prec1<-diag(REn)

   probs<-v<-matrix(.5,REn,J);
   v[,J]<-1
   for(j in 1:REn){probs[j,]<-makeprobs(v[j,],J)}
   g<-round(matrix(runif(REn*nt,0.5,J+0.5),REn,nt))
   D<-1

   probsh<-vh<-matrix(.5,ntooth,J);
   vh[,J]<-1
   for(j in 1:ntooth){probsh[j,]<-makeprobs(vh[j,],J)}
   h<-round(matrix(runif(ns*nt,0.5,J+0.5),ns,nt))
   mnh<-matrix(0,ntooth,J)
   tauh<-rep(1,ntooth)
   Dh<-1
   

   grid<-seq(-10,10,.05)
   ngrid<-length(grid)
   D1<-D2<-matrix(0,ngrid,REn)
   D3<-D4<-matrix(0,ngrid,ntooth)


   alpha1<-alpha2<-theta1<-theta2<-matrix(0,ns,nt)
   mnalpha1<-0*alpha1
   preca1<-preca2<-rep(1,ntooth)

   keep.beta<-keep.beta2<-matrix(0,iters,p)
   keep.mnz<-matrix(0,iters,REn)
   keep.rho<-keep.a<-keep.power<-keep.ratio<-keep.sige<-matrix(0,iters,ntooth)

   Z1<-FE
   Z2<-ZMISS%*%FE
   Z3<-ZMISS

   Z1RE1<-Z1%*%RE1
   AZ2RE1<-A%*%Z2%*%RE1
   Z2RE2<-Z2%*%RE2
   AZ3t1<-A%*%Z3%*%theta1
   Z3t2<-Z3%*%theta2


   for(t in 1:nt){
      RE1[,t]<-lm(y[,t]~Z1-1)$coef
      RE2[,t]<-lm(z[,t]~Z2-1)$coef
   }
   RE1[is.na(RE1)]<-0
   RE2[is.na(RE2)]<-0
   if(!infomiss){RE2<-0*RE2}
#   keep.COV<-array(0,c(8,8,iters))

   tXX<-t(X)%*%X
   XB1<-matrix(X%*%beta1,ns,nt,byrow=T)
   XB2<-matrix(X%*%beta2,nm,nt,byrow=T)
   AZ3XB1<-A%*%Z3%*%XB1

   summn<-summn2<-0*y

   RE1mng<-matrix(0,REn,nt)
   for(t in 1:nt){for(l in 1:REn){
      RE1mng[l,t]<-RE1mn[l,g[l,t]]
   }}


   pred.y<-pred.z<-pred.spatial<-pred.RE<-NULL
   if(post.pred){
      pred.y<-pred.spatial<-matrix(0,iters,ns)
      pred.RE<-matrix(0,iters,REn)
      pred.z<-matrix(0,iters,nm)
   }

   RESP1<-RESP2<-matrix(0,nt,9)
   colnames(RESP1)<-colnames(RESP2)<-
       c("mean","logsd","IQR","P>3","P>5","P>7","Q10","Q50","Q90")

   for(i in 1:iters){

     mn1<-XB1+Z1RE1+theta1
     mn2<-XB2+Z2RE2+Z3t2+AZ3XB1+AZ2RE1+AZ3t1

     #update missings:
     sige<-1/sqrt(taue[tooth])
     for(t in 1:nt){
       ynew<-rnorm(ns,mn1[,t],sige)
       y[,t]<-ifelse(miss[,t],ynew,y[,t])
       if(infomiss){
         z[,t]<-rtnorm(nm,mn2[,t],rep(1,nm),L[,t],U[,t])
       }
     }

     R<-y-mn1
     taue[1:ntooth]<-rgamma(1,sum(!miss)/2+0.1,sum(R[!miss]^2)/2+.1)
     if(nonstat){for(j in 1:ntooth){
        RRR<-R[tooth==j,]
        mmm<-miss[tooth==j,]
        taue[j]<-rgamma(1,sum(!mmm)/2+.1,sum(RRR[!mmm]^2)/2+.1)
     }}

     ##################################################:
     ########       UPDATE FIXED EFFECTS     ##########:
     ##################################################:

     VVV<-0.01*diag(p)
     MMM<-rep(0,p)
     RRR<-y-Z1RE1-theta1
     R2<-z-XB2-Z2RE2-Z3t2-AZ2RE1-AZ3t1
     AZ3<-A%*%Z3
     for(t in 1:nt){
        TTT<-taue[tooth]*(!miss[,t])
        XXX<-matrix(X[t,],ns,p,byrow=T)
        VVV<-VVV+t(XXX)%*%diag(TTT)%*%XXX
        MMM<-MMM+t(XXX)%*%(TTT*RRR[,t])

        if(infomiss){
           XXX<-AZ3%*%matrix(X[t,],ns,p,byrow=T)
           VVV<-VVV+t(XXX)%*%XXX
           MMM<-MMM+t(XXX)%*%R2[,t]
        }

     }
     VVV<-solve(VVV)
     beta1<-VVV%*%MMM+t(chol(VVV))%*%rnorm(p)
     XB1<-matrix(X%*%beta1,ns,nt,byrow=T)
     AZ3XB1<-AZ3%*%XB1

     if(infomiss){
      VVV<-0.01*diag(p+REn)
      MMM<-rep(0,p+REn)
      RRR<-z-Z2RE2-Z3t2-AZ3XB1-AZ2RE1-AZ3t1
      for(t in 1:nt){
       XXX<-cbind(matrix(X[t,],nm,p,byrow=T),Z2)
       VVV<-VVV+t(XXX)%*%XXX
       MMM<-MMM+t(XXX)%*%RRR[,t]
      }
      VVV<-solve(VVV)
      bbb<-VVV%*%MMM+t(chol(VVV))%*%rnorm(p+REn)

      for(t in 1:nt){
       XXX<-cbind(matrix(X[t,],nm,p,byrow=T),Z2)
       XB2[,t]<-XXX%*%bbb
      }
      beta2<-bbb[1:p]
      RE2mn<-bbb[1:REn+p]
     }

     ##################################################:
     ########       UPDATE GROUP LABELS      ##########:
     ##################################################:

     if(!commong & J>1){
      for(t in 1:nt){for(l in 1:REn){     
         RRR<-log(probs[l,])
         ddd<-rrr<-RE1[,t]-RE1mng[,t]
         rrr[l]<-rrr[l]+RE1mng[l,t]
         for(j in 1:J){
            ddd[l]<-rrr[l]-RE1mn[l,j]
            RRR[j]<-RRR[j]-0.5*t(ddd)%*%RE1prec1%*%ddd
         }
         RRR<-exp(RRR-max(RRR))
         RRR[is.na(RRR)]<-0
         if(sum(RRR)>0){g[l,t]<-sample(1:J,1,prob=RRR)}
         RE1mng[l,t]<-RE1mn[l,g[l,t]]
      }}


      for(l in 1:REn){
       v[l,]<-new.v(g[l,],D,J)
       probs[l,]<-makeprobs(v[l,],J)
      }
      D<-new.b(v[,-J]) 
      D<-1
    }



     if(commong & J>1){
      ZZZ<-rbind(Z1,A%*%Z2)
      R1<-y-XB1-theta1
      R2<-z-Z2RE2-Z3t2-AZ3XB1-AZ3t1
      for(t in 1:nt){     
         TTT<-diag(c(taue[tooth]*(!miss[,t]),rep(1,nm)))
         OOO<-solve(t(ZZZ)%*%TTT%*%(ZZZ)+RE1prec1)
         YYY<-c(R1[,t],R2[,t])
         RRR<-log(probs[1,])
         ZTY<-t(ZZZ)%*%TTT%*%YYY
         for(j in 1:J){     
           MMM<-RE1prec1%*%RE1mn[,j]
           SSS<-sum(MMM*RE1mn[,j])
           RRR[j]<-RRR[j]-0.5*SSS+
                   0.5*t(ZTY+MMM)%*%OOO%*%(ZTY+MMM)
         }
         RRR<-exp(RRR-max(RRR))
         RRR[is.na(RRR)]<-0
         if(sum(RRR)>0){g[,t]<-sample(1:J,1,prob=RRR)}
         RE1mng[,t]<-RE1mn[,g[1,t]]
       }

       v[1,]<-new.v(g[1,],D,J)
       probs[1,]<-makeprobs(v[1,],J)
       for(l in 2:REn){probs[l,]<-probs[1,]}
       D<-new.b(v[1,-J]) 
       D<-1
     }


     ##################################################:
     ########      UPDATE RANDOM EFFECTS     ##########:
     ##################################################:

     oldRE1<-RE1
     oldRE2<-RE2

     if(infomiss){
        AZ2<-A%*%Z2
        RZ<-z-XB2-Z2RE2-Z3t2-AZ3XB1-AZ3t1
        tZ2Z2<-t(AZ2)%*%AZ2
        tZ2r<-t(AZ2)%*%RZ
     }


      for(t in 1:nt){
        TTT<-diag(c(taue[tooth]*(!miss[,t])))
        VVV<-t(Z1)%*%TTT%*%Z1+RE1prec1
        MMM<-t(Z1)%*%TTT%*%(y[,t]-XB1[,t]-theta1[,t])+
             RE1prec1%*%RE1mng[,t]
        if(infomiss){
          VVV<-VVV+tZ2Z2
          MMM<-MMM+tZ2r[,t]
        }
        VVV<-solve(VVV)
        RE1[,t]<-VVV%*%MMM+t(chol(VVV))%*%rnorm(REn)
      }
      RE1[is.na(RE1)]<-oldRE1[is.na(RE1)]
      Z1RE1<-Z1%*%RE1
      AZ2RE1<-A%*%Z2%*%RE1
   

      if(infomiss & REmissing){
        VVV<-solve(t(Z2)%*%Z2+RE2prec1)
        RRR<-z-XB2-Z3t2-AZ3XB1-AZ2RE1-AZ3t1
        RE2<-VVV%*%t(Z2)%*%RRR+
             t(chol(VVV))%*%matrix(rnorm(REn*nt),REn,nt) 
        RE2[is.na(RE2)]<-oldRE2[is.na(RE2)]
        Z2RE2<-Z2%*%RE2
      }

      if(infomiss){ 
        RR<-z-XB2-Z2RE2-Z3t2
        XX<-Z3%*%XB1+Z2%*%RE1+Z3%*%theta1

        VVV<-1/(sum(XX^2)+0.01)
        MMM<-sum(XX*RR)
        Avec[1:ntooth]<-rnorm(1,MMM*VVV,sqrt(VVV))

        if(nonstat){for(j in 1:ntooth){  
          RRR<-RR[toothz==j,]
          XXX<-XX[toothz==j,]
          VVV<-1/(sum(XXX^2)+0.01)
          MMM<-sum(XXX*RRR)
          if(!is.na(MMM+VVV)){if(abs(sum(MMM+VVV))<Inf){
            Avec[j]<-rnorm(1,MMM*VVV,sqrt(VVV))
          }}
        }}
        A<-diag(Avec[toothz])
        AZ2RE1<-A%*%Z2%*%RE1
        AZ3t1<-A%*%Z3%*%theta1
        AZ3XB1<-A%*%Z3%*%XB1
     } 

     ##################################################:
     ########          SPATIAL TERMS         ##########:
     ##################################################:

     oldalpha1<-alpha1

     if(infomiss){
        AZ3<-A%*%Z3%*%W
        RZ<-z-XB2-Z2RE2-Z3t2-AZ3XB1-AZ2RE1
        tZ3Z3<-t(AZ3)%*%AZ3
        tZ3r<-t(AZ3)%*%RZ
     }

     if(spatial){
      R<-y-XB1-Z1RE1
      
      for(t in 1:nt){
        TTT<-sweep(t(W),2,taue[tooth]*(!miss[,t]),"*")
        VVV<-TTT%*%W+diag(preca1[tooth])
        MMM<-TTT%*%R[,t]+preca1[tooth]*mnalpha1[,t]
        if(infomiss){
          VVV<-VVV+tZ3Z3
          MMM<-MMM+tZ3r[,t]
        }
        VVV<-solve(VVV)
        alpha1[,t]<-VVV%*%MMM+t(chol(VVV))%*%rnorm(ns)
       }
       alpha1[is.na(alpha1)]<-oldalpha1[is.na(alpha1)]
       theta1<-W%*%alpha1
       AZ3t1<-A%*%Z3%*%theta1

     }

     if(F & infomiss & spatial & REmissing){
      oldalpha2<-alpha2
      R<-z-XB2-Z2RE2-AZ3XB1-AZ2RE1-AZ3t1
      ZZ<-Z3%*%W
      VVV<-solve(t(ZZ)%*%ZZ+diag(preca2[tooth]))

      MMM<-t(ZZ)%*%R
      alpha2<-VVV%*%MMM+
              t(chol(VVV))%*%matrix(rnorm(ns*nt),ns,nt)
      alpha2[is.na(alpha2)]<-oldalpha2[is.na(alpha2)]
      theta2<-W%*%alpha2
      Z3t2<-Z3%*%theta2
     }


    if(spatial & NPspatial){
      for(j in 1:ns){
        ppp<-probsh[tooth[j],]
        mmm<-mnh[tooth[j],]
        sss<-1/sqrt(preca1[tooth[j]])
        for(t in 1:nt){
           RRR<-ppp*dnorm(alpha1[j,t],mmm,sss)
           h[j,t]<-sample(1:J,1,prob=RRR)
           mnalpha1[j,t]<-mmm[h[j,t]]
        }
      }  

      for(l in 1:ntooth){
       vh[l,]<-new.v(h[tooth==l,],Dh,J)
       probsh[l,]<-makeprobs(vh[l,],J)
      }
      Dh<-new.b(vh[,-J]) 
      Dh<-1

      for(l in 1:ntooth){
        yyy<-alpha1[tooth==l,]
        hhh<-h[tooth==l,]
        SS<-0
        for(j in 1:J){
          VVV<-tauh[l]+sum(hhh==j)*preca1[l]
          MMM<-preca1[l]*sum(yyy[hhh==j])
          mnh[l,j]<-rnorm(1,MMM/VVV,1/sqrt(VVV))
          SS<-SS+sum((yyy[hhh==j]-mnh[l,j])^2)
        }
        tauh[l]<-rgamma(1,sum(hhh>0)/2+.1,SS/2+.1)
      }            
      for(j in 1:ns){for(t in 1:nt){
        mnalpha1[j,t]<-mnh[tooth[j],h[j,t]]
      }}
     }   

     ##################################################:
     ########        Y HYPERPARAMETERS       ##########:
     ##################################################:
      RRR<-RE1
      if(commong){for(j in 1:J){
       VVV<-solve(sum(g[1,]==j)*RE1prec1+RE1prec2)
       MMM<-RE1prec2%*%RE1mnmn
       if(sum(g[1,]==j)==1){MMM<-MMM+RE1prec1%*%RRR[,g[1,]==j]}
       if(sum(g[1,]==j)>1){MMM<-MMM+RE1prec1%*%rowSums(RRR[,g[1,]==j])}
       RE1mn[,j]<-VVV%*%MMM+t(chol(VVV))%*%rnorm(REn)
       for(t in 1:nt){if(g[1,t]==j){RE1mng[,t]<-RE1mn[,j]}}
      }}

      if(!commong){for(j in 1:J){for(l in 1:REn){
       VVV<-sum(g[l,]==j)*RE1prec1[l,l]+RE1prec2[l,l]
       MMM<-RE1prec2[l,l]*RE1mnmn[l]-
            RE1prec2[l,-l]%*%(RE1mn[-l,j]-RE1mnmn[-l])
       for(t in 1:nt){if(g[l,t]==j){     
         these<-cbind(1:REn,g[,t]) 
         MMM<-MMM+RE1prec1[l,l]*RRR[l,t]+
                  RE1prec1[l,-l]%*%(RRR[-l,t]-RE1mn[these[-l,]])
       }}           
       RE1mn[l,j]<-rnorm(1,MMM/VVV,1/sqrt(VVV))
       RE1mng[l,g[l,]==j]<-RE1mn[l,j]
      }}}


#      ddd<-t(RE1-RE1mng)
#      RE1prec1<-update.wish(ddd,1,1*diag(REn),nt,REn)
      RE1prec1<-diag(REn)
      for(j in 1:REn){
        RE1prec1[j,j]<-rgamma(1,nt/2+.5,sum((RE1[j,]-RE1mng[j,])^2)/2+.005)
      }

      if(J==1){
         RE1prec2<-0.01*diag(REn)
         RE1mnmn<-rep(0,REn)
      }
      
      if(J>1){
       VVV<-solve(J*RE1prec2+0.01*diag(REn))
       MMM<-RE1prec2%*%rowSums(RE1mn)
       RE1mnmn<-VVV%*%MMM+t(chol(VVV))%*%rnorm(REn)
 
       if(commong){
         ddd<-t(RE1mn-matrix(RE1mnmn,REn,J,byrow=T))
         RE1prec2<-update.wish(ddd,.1,solve(0.5*diag(REn)+.5),J,REn)
       }
       if(!commong){
         RE1prec2<-diag(REn)
         for(j in 1:REn){
          RE1prec2[j,j]<-rgamma(1,J/2+1,sum((RE1mn[j,]-RE1mnmn[j])^2)/2+1)
         }
       }
 
      }


     ##################################################:
     ########        Z HYPERPARAMETERS       ##########:
     ##################################################:

     if(infomiss){
       ddd<-t(RE2)
       RE2prec1<-update.wish(ddd,1,1*diag(REn),nt,REn)
     }



     ##################################################:
     ########     SPATIAL HYPERPARAMETERS    ##########:
     ##################################################:

     for(j in 1:ntooth){
       nnn<-sum(tooth==j)*nt
       SSS<-sum((alpha1[tooth==j,]-mnalpha1[tooth==j,])^2)
       preca1[j]<-rgamma(1,nnn/2+.1,SSS/2+.1)
       SSS<-sum(alpha2[tooth==j,])
       preca2[j]<-rgamma(1,nnn/2+.1,SSS/2+.1)  
     }


     R1<-y-XB1-Z1RE1
     R2<-z-XB2-Z2RE2-Z3t2-AZ3XB1-AZ2RE1
     AZ3<-A%*%Z3     
     TTT<-matrix(taue[tooth],ns,nt,byrow=F)

     R1o<-sum(TTT[!miss]*(R1[!miss]-theta1[!miss])^2)
     R2o<-R2n<-sum((R2-AZ3t1)^2)

#DOESN'T ACCOUNT FOR THETA2
     if(spatial & !nonstat){
        canratio<-ratio
        canratio[1:ntooth]<-exp(rnorm(1,log(ratio[1]),0.02))
        canW<-MakeW(d1,d2,canratio[tooth],rho[tooth],power[tooth])
        cantheta1<-canW%*%alpha1
        R1n<-sum(TTT[!miss]*(R1[!miss]-cantheta1[!miss])^2)
        R<-dnorm(log(canratio[1]),0,10,log=T)-
           dnorm(log(ratio[1]),0,10,log=T)-
           0.5*R1n+0.5*R1o
        if(infomiss){
          R2n<-sum((R2-AZ3%*%cantheta1)^2)
          R<-R-0.5*R2n+0.5*R2o
        }
        if(!is.na(exp(R))){if(runif(1)<exp(R)){
          ratio<-canratio
          W<-canW
          theta1<-cantheta1
          R1o<-R1n
          R2o<-R2n 
        }}



        canrho<-rho
        canrho[1:ntooth]<-exp(rnorm(1,log(rho[1]),0.02))
        canW<-MakeW(d1,d2,ratio[tooth],canrho[tooth],power[tooth])
        cantheta1<-canW%*%alpha1
        R1n<-sum(TTT[!miss]*(R1[!miss]-cantheta1[!miss])^2)
        R<-dnorm(log(canrho[1]),0,10,log=T)-
           dnorm(log(rho[1]),0,10,log=T)-
           0.5*R1n+0.5*R1o
        if(infomiss){
          R2n<-sum((R2-AZ3%*%cantheta1)^2)
          R<-R-0.5*R2n+0.5*R2o
        }
        if(!is.na(exp(R))){if(runif(1)<exp(R)){
          rho<-canrho
          W<-canW
          theta1<-cantheta1
          R1o<-R1n
          R2o<-R2n 
        }}


       if(F){
        canpower<-power
        canpower[1:ntooth]<-exp(rnorm(1,log(power[1]),0.02))
        canW<-MakeW(d1,d2,ratio[tooth],rho[tooth],canpower[tooth])
        cantheta1<-canW%*%alpha1
        R1n<-sum(TTT[!miss]*(R1[!miss]-cantheta1[!miss])^2)
        R<-dnorm(log(canpower[1]),0,10,log=T)-
           dnorm(log(power[1]),0,10,log=T)-
           0.5*R1n+0.5*R1o
        if(infomiss){
          R2n<-sum((R2-AZ3%*%cantheta1)^2)
          R<-R-0.5*R2n+0.5*R2o
        }
        if(!is.na(exp(R))){if(runif(1)<exp(R)){
          power<-canpower
          W<-canW
          theta1<-cantheta1
          R1o<-R1n
          R2o<-R2n 
        }}
       }
     }


     if(spatial & nonstat){for(j in 1:ntooth){
        canratio<-ratio
        canratio[j]<-exp(rnorm(1,log(ratio[j]),0.025))
        canW<-MakeW(d1,d2,canratio[tooth],rho[tooth],power[tooth])
        cantheta1<-canW%*%alpha1
        R1n<-sum(TTT[!miss]*(R1[!miss]-cantheta1[!miss])^2)
        R<-dnorm(log(canratio[j]),0,10,log=T)-
           dnorm(log(ratio[j]),0,10,log=T)-
           0.5*R1n+0.5*R1o
        if(infomiss){
          R2n<-sum((R2-AZ3%*%cantheta1)^2)
          R<-R-0.5*R2n+0.5*R2o
        }
        if(!is.na(exp(R))){if(runif(1)<exp(R)){
          ratio<-canratio
          W<-canW
          theta1<-cantheta1
          R1o<-R1n
          R2o<-R2n 
        }}



        canrho<-rho
        canrho[j]<-exp(rnorm(1,log(rho[j]),0.025))
        canW<-MakeW(d1,d2,ratio[tooth],canrho[tooth],power[tooth])
        cantheta1<-canW%*%alpha1
        R1n<-sum(TTT[!miss]*(R1[!miss]-cantheta1[!miss])^2)
        R<-dnorm(log(canrho[j]),0,10,log=T)-
           dnorm(log(rho[j]),0,10,log=T)-
           0.5*R1n+0.5*R1o
        if(infomiss){
          R2n<-sum((R2-AZ3%*%cantheta1)^2)
          R<-R-0.5*R2n+0.5*R2o
        }

        if(!is.na(exp(R))){if(runif(1)<exp(R)){
          rho<-canrho
          W<-canW
          theta1<-cantheta1
          R1o<-R1n
          R2o<-R2n 
        }}


     if(F){
        canpower<-power
        canpower[j]<-exp(rnorm(1,log(power[j]),0.05))
        canW<-MakeW(d1,d2,ratio[tooth],rho[tooth],canpower[tooth])
        cantheta1<-canW%*%alpha1
        R1n<-sum(TTT[!miss]*(R1[!miss]-cantheta1[!miss])^2)
        R<-dnorm(log(canpower[j]),0,10,log=T)-
           dnorm(log(power[j]),0,10,log=T)-
           0.5*R1n+0.5*R1o
        if(infomiss){
          R2n<-sum((R2-AZ3%*%cantheta1)^2)
          R<-R-0.5*R2n+0.5*R2o
        }
        if(!is.na(exp(R))){if(runif(1)<exp(R)){
          power<-canpower
          W<-canW
          theta1<-cantheta1
          R1o<-R1n
          R2o<-R2n 
        }}
     }
     }}
     AZ3t1<-AZ3%*%theta1

     ##################################################:
     ########       KEEP TRACK OF STUFF      ##########:
     ##################################################:

      mn1<-XB1+Z1RE1+theta1
      if(i>burn){
        summn<-summn+mn1/(iters-burn)
        summn2<-summn2+(mn1^2)/(iters-burn)
       }


       keep.beta[i,]<-beta1     
       keep.beta2[i,]<-beta2     
       keep.rho[i,]<-rho
       keep.power[i,]<-power
       keep.ratio[i,]<-ratio
       keep.a[i,]<-Avec
       keep.sige[i,]<-1/sqrt(taue)
       keep.mnz[i,]<-RE2mn

       if(i>burn){
        for(l in 1:REn){
         ddd<-rep(0,ngrid)
         sss<-sqrt(solve(RE1prec1)[l,l])
         for(j in 1:J){
           ddd<-ddd+probs[l,j]*dnorm(grid,RE1mn[l,j],sss)
         }  
         ddd<-ddd/sum(ddd)           
         D1[,l]<-D1[,l]+ddd/(iters-burn)
         D2[,l]<-D2[,l]+ddd*ddd/(iters-burn)
        }

        for(l in 1:ntooth){
         ddd<-rep(0,ngrid)
         sss<-sqrt(1/preca1[l])
         for(j in 1:J){
           ddd<-ddd+probsh[l,j]*dnorm(grid,mnh[l,j],sss)
         }  
         ddd<-ddd/sum(ddd)           
         D3[,l]<-D3[,l]+ddd/(iters-burn)
         D4[,l]<-D4[,l]+ddd*ddd/(iters-burn)
        }

        rrr<-apply(mn1,2,mean)
        RESP1[,1]<-RESP1[,1]+rrr/(iters-burn)
        RESP2[,1]<-RESP2[,1]+rrr/(iters-burn)

        rrr<-log(apply(mn1,2,sd))
        RESP1[,2]<-RESP1[,2]+rrr/(iters-burn)
        RESP2[,2]<-RESP2[,2]+rrr*rrr/(iters-burn)

        rrr<-log(apply(mn1,2,quantile,.75)-apply(mn1,2,quantile,0.25))
        RESP1[,3]<-RESP1[,3]+rrr/(iters-burn)
        RESP2[,3]<-RESP2[,3]+rrr*rrr/(iters-burn)


        rrr<-apply(mn1>3,2,mean)
        RESP1[,4]<-RESP1[,4]+rrr/(iters-burn)
        RESP2[,4]<-RESP2[,4]+rrr*rrr/(iters-burn)


        rrr<-apply(mn1>5,2,mean)
        RESP1[,5]<-RESP1[,5]+rrr/(iters-burn)
        RESP2[,5]<-RESP2[,5]+rrr*rrr/(iters-burn)


        rrr<-apply(mn1>7,2,mean)
        RESP1[,6]<-RESP1[,6]+rrr/(iters-burn)
        RESP2[,6]<-RESP2[,6]+rrr*rrr/(iters-burn)


        rrr<-apply(mn1,2,quantile,.1)
        RESP1[,7]<-RESP1[,7]+rrr/(iters-burn)
        RESP2[,7]<-RESP2[,7]+rrr*rrr/(iters-burn)


        rrr<-apply(mn1,2,quantile,.5)
        RESP1[,8]<-RESP1[,8]+rrr/(iters-burn)
        RESP2[,8]<-RESP2[,8]+rrr*rrr/(iters-burn)


        rrr<-apply(mn1,2,quantile,.9)
        RESP1[,9]<-RESP1[,9]+rrr/(iters-burn)
        RESP2[,9]<-RESP2[,9]+rrr*rrr/(iters-burn)


       }
       if(post.pred){
         pred.y[i,]<-y[,nt]
         pred.z[i,]<-z[,nt]
         pred.spatial[i,]<-theta1[,nt]
         pred.RE[i,]<-RE1[,nt]
       }



       #display current iteration
       if(i%%display==0){
         par(mfrow=c(3,2))
         plot(y[,1],col=miss[,1]+1,main=g[1])
         lines(mn1[,1])
         plot(g[4,],RE1[4,])


         hhh<-hist(RE1[3,],breaks=25)$counts
         for(j in 1:J){lines(rep(RE1mn[3,j],2),c(0,max(hhh)*probs[3,j]),col=2,lwd=2)}
         j<-3
         cang<-sample(1:J,10000,replace=T,prob=probs[j,])
         samp<-rnorm(10000,RE1mn[j,cang],sqrt(solve(RE1prec1)[j,j]))
         hist(samp,breaks=100)

         plot(keep.beta[1:i,1],type="l")
         abline(0,0)

         plot(keep.rho[1:i,4],type="l")
        }
     
   }

if(post.pred){
  RESP1<-RESP1[-nt,]
  RESP2<-RESP2[-nt,]
}

list(fit.mn=summn,
     fit.var=summn2-summn^2,
     beta=keep.beta,
     beta2=keep.beta2,
     rho=keep.rho,
     power=keep.power,
     a=keep.a,
     mnz=keep.mnz,
     ratio=keep.ratio,
     sige=keep.sige,
     RE.dense.mn=D1,
     RE.dense.var=D2-D1^2,
     spatial.dense.mn=D3,
     spatial.dense.var=D4-D3^2,
     grid=grid,
     pred.y=pred.y,
     pred.z=pred.z,
     pred.RE=pred.RE,
     pred.spatial=pred.spatial,
     RESP.mn=RESP1,
     RESP.var=RESP2-RESP1^2
)}
mpi.bcast.Robj2slave(KC)










